import os.path

BOOKS_DIR = "data" # Название папки с книгами
BOOKS_FILE = os.path.join(BOOKS_DIR, "files.csv") # Имя файла со списком книг


def books_list():
    """
    Returns list of books from a file.csv.
    List contains tuples: (Author, Title, book_id, Annotation text)
    """
    result = [] # [(Автор, Название, id, Аннотация), ...]
    with open(BOOKS_FILE, "r", encoding="utf-8") as f:
        for line in f:
            if line.startswith("#"): # Пропускаем все строки, начинающиеся с #
                continue
            line = line.strip() # Чистим строку от лишних пробелов
            data = line.split(";", maxsplit=4) # Режем строку по разделителям
            result.append(tuple(data)) # Складываем результаты в список
    return result


def find_book(book_id):
    """
    Returns tuple (author, title, book_id, annotation) for a book with given book_id.
    Raises KeyError if book with given id was not found.
    """
    books = books_list() # Получаем список книг из файла
    for book in books: # Перебираем книги
        author, title, id, annot = book # Распаковываем кортеж в переменные
        if id == book_id: # Если нашли нужную книгу...
            return book # ...возвращаем её
    raise KeyError(f"Book with id '{book_id}' not found") # Если ничего не нашли - бросаем исключение


def get_book_text(book_id):
    """
    Returns list of text lines of the given book.
    """
    result = []
    filename = os.path.join(BOOKS_DIR, book_id + ".txt") # Собираем имя файла книги
    with open(filename, "r", encoding="utf-8") as f:
        for line in f:
            if line.endswith("\n"): # Убираем символ перевода строки, если он там есть
                line = line[:-1]
            if len(line.strip()) > 0: # Если строка не пустая и не состоит из одних только пробелов...
                result.append(line) # Добавляем строку в результат
    return result


def search(text, only_annot, lines_per_page):
    """
    Performs a simple search of given text trough all the books.
    Parameters:
        text           - text to search
        only_annot     - if True will not search in the books' text
        lines_per_page - number of lines per screen page to calculate the number of page with the search result
    """
    results = []
    books = books_list() # Получаем список книг
    for author, title, book_id, annot in books:
        if text in annot: # Если искомый текст в аннотации
            results.append({ # Добавляем результат поиска
                "book_id": book_id,
                "title": title,
                "page": 0,
                "text": annot
            })
        if not only_annot: # Если ищем не только по аннотациям...
            lines = get_book_text(book_id) # Загружаем текст книги
            for index, line in enumerate(lines): # Перебираем все строки вместе с их номерами (то есть индексами)
                if text in line: # Если искомый текст есть в текущей строке,
                    page = index // lines_per_page # Вычисляем номер страницы, на которой эта строка
                    results.append({ # Добавляем результат поиска
                        "book_id": book_id,
                        "title": title,
                        "page": page,
                        "text": line
                    })
    return results